/*---------------------------------------------------------------------------*
  Project:  GAMECUBE DSP ADPCM win32 DLL
  File:     dspadpcm.h

  Copyright 1998, 1999, 2000, 2001 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#define f32 float
#define u8	unsigned char
#define s16 signed short
#define u16 unsigned short
#define s32 signed long
#define u32 unsigned long

#ifndef __DSPADPCM_H__
#define __DSPADPCM_H__

//#define MAKE_A_DLL

#ifdef MAKE_A_DLL
#define LINKDLL __declspec(dllexport)
#else
#define LINKDLL __declspec(dllimport)
#endif

/*---------------------------------------------------------------------------*
    ADPCM info passed to the caller
 *---------------------------------------------------------------------------*/
typedef struct
{
    // start context
    s16 coef[16];
    u16 gain;
    u16 pred_scale;
    s16 yn1;
    s16 yn2;

    // loop context
    u16 loop_pred_scale;
    s16 loop_yn1;
    s16 loop_yn2;

} ADPCMINFO;

// all data in this structure is in BIG ENDIAN FORMAT!!!!
typedef struct
{         
     // for header generation during decode
     u32 num_samples;         //0 total number of RAW samples
     u32 num_adpcm_nibbles;   //4 number of ADPCM nibbles (including frame headers)
     u32 sample_rate;         //8 Sample rate, in Hz
     // DSP addressing and decode context
     u16 loop_flag;           //C 1=LOOPED, 0=NOT LOOPED
     u16 format;              //E Always 0x0000, for ADPCM
     u32 sa;                  //10 Start offset address for looped samples (zero for non-looped)
     u32 ea;                  //14 End offset address for looped samples
     u32 ca;                  //18 always zero
     u16 coef[16];            //1C decode coefficients (eight pairs of 16-bit words)  
     // DSP decoder initial state
     u16 gain;                //3C always zero for ADPCM
     u16 ps;                  //3E predictor/scale
     u16 yn1;                 //40 sample history
     u16 yn2;                 //42 sample history
     // DSP decoder loop context
     u16 lps;                 //44 predictor/scale for loop context
     u16 lyn1;                //46 sample history (n-1) for loop context
     u16 lyn2;                //48 sample history (n-2) for loop context
     u16 pad[11];             //4A reserved
     } sDSPADPCM;
     // Header is 96 bytes long

u32 getBytesForPcmBuffer        (u32 samples);

int decode
(
    u8          *src,   // location of encoded source samples
    s16         *dst,   // location of destination buffer (16 bits / sample)
    ADPCMINFO   *cxt,   // location of adpcm info
    u32         samples // number of samples to decode         
);

int decodeFILE
(
    HANDLE input_file,   // file handle of encoded source samples
    s16         *dst,   // location of destination buffer (16 bits / sample)
    ADPCMINFO   *cxt,   // location of adpcm info
    u32         samples, // number of samples to decode         
	short *, short *
);

int decodeFILEalt
(
    HANDLE input_file,   // file handle of encoded source samples
    s16         *dst,   // location of destination buffer (16 bits / sample)
    ADPCMINFO   *cxt,   // location of adpcm info
    u32         samples, // number of samples to decode         
	short *, short *, short *
);

/*---------------------------------------------------------------------------*
    exported functions
 *---------------------------------------------------------------------------*/

u32 getBytesForAdpcmBuffer      (u32 samples);
u32 getBytesForAdpcmSamples     (u32 samples);
u32 getBytesForPcmBuffer        (u32 samples);
u32 getBytesForPcmSamples       (u32 samples);
u32 getSampleForAdpcmNibble     (u32 nibble);
u32 getBytesForAdpcmInfo        (void);
u32 getNibbleAddress            (u32 samples);
/*
void encode
(
    s16         *src,   // location of source samples (16bit PCM signed little endian)
    u8          *dst,   // location of destination buffer
    ADPCMINFO   *cxt,   // location of adpcm info
    u32         samples // number of samples to encode         
);

void decode
(
    u8          *src,   // location of encoded source samples
    s16         *dst,   // location of destination buffer (16 bits / sample)
    ADPCMINFO   *cxt,   // location of adpcm info
    u32         samples // number of samples to decode         
);

void getLoopContext
(
    u8          *src,      // location of ADPCM buffer in RAM
    ADPCMINFO   *cxt,      // location of adpcminfo
    u32         samples    // samples to desired context
);
*/
#endif